// BLACK FOREST - Written by Fast_n_Painless.

void main(void)

{

// Text
   rmSetStatusText("",0.01);

// Set size.
   int playerTiles= 8500; 
   if(cMapSize == 1)
   {
      playerTiles = 12500; 
      rmEchoInfo("Large map");
   }
   int size=2.0*sqrt(cNumberNonGaiaPlayers*playerTiles/0.9);
   rmEchoInfo("Map size="+size+"m x "+size+"m");
   rmSetMapSize(size, size);

// Set up default water.
   rmSetSeaLevel(0.0);

// Init map.
   rmTerrainInitialize("GrassA"); 
    
// ----- Define some classes.
   int classPlayer=rmDefineClass("player");
   int classConnection=rmDefineClass("team connection");
   int classOakForest=rmDefineClass("oak forest");
   int classMixedOakForest=rmDefineClass("mixed oak forest");
   int classPineForest=rmDefineClass("pine forest");
   int classPond=rmDefineClass("pond");
   rmDefineClass("starting settlement");
   rmDefineClass("far settlement");

// ----- Define constraints.

// Create edge of map constraint.
   int edgeConstraint=rmCreateBoxConstraint("edge of map", rmXTilesToFraction(8), rmZTilesToFraction(8), 1.0-rmXTilesToFraction(8), 1.0-rmZTilesToFraction(8));

// Player constraint.
   int playerConstraint=rmCreateClassDistanceConstraint("stay away from players", classPlayer, 20);

// Settlement constraints.
   int shortAvoidSettlement=rmCreateTypeDistanceConstraint("objects avoid TC by short distance", "AbstractSettlement", 20.0);
   int farAvoidSettlement=rmCreateTypeDistanceConstraint("objects avoid TC by long distance", "AbstractSettlement", 50.0);
   int farStartingSettleConstraint=rmCreateClassDistanceConstraint("objects avoid player TCs", rmClassID("starting settlement"), 50.0);

// Connection.
   int avoidConnection=rmCreateClassDistanceConstraint("stay away from connection", rmClassID("team connection"), 5);

// Tower constraint.
   int avoidTower=rmCreateTypeDistanceConstraint("towers avoid towers", "tower", 20.0);
   int avoidTower2=rmCreateTypeDistanceConstraint("objects avoid towers", "tower", 22.0);

// Gold.
   int avoidGold=rmCreateTypeDistanceConstraint("avoid gold", "gold", 30.0);
   int shortAvoidGold=rmCreateTypeDistanceConstraint("gold vs. gold", "gold", 24.0);

// Food.
   int avoidHerdable=rmCreateTypeDistanceConstraint("avoid herdable", "herdable", 20.0);
   int avoidFood=rmCreateTypeDistanceConstraint("avoid food", "food", 12.0);
   int avoidPredator=rmCreateTypeDistanceConstraint("avoid predator", "animalPredator", 40.0);

// Fish.
   int fishVsFishID=rmCreateTypeDistanceConstraint("fish v fish", "fish", 5.0);
   int fishLand = rmCreateTerrainDistanceConstraint("fish land", "land", true, 8.0);

// Huntable.
   int classBonusHuntable=rmDefineClass("bonus huntable");
   int avoidBonusHuntable=rmCreateClassDistanceConstraint("avoid bonus huntable", classBonusHuntable, 40.0);
   int avoidHuntable=rmCreateTypeDistanceConstraint("avoid huntable", "huntable", 20.0);

// Pond.
   int pondConstraint=rmCreateClassDistanceConstraint("pond vs. pond", rmClassID("pond"), 10.0);

// Avoid impassable land.
   int avoidImpassableLand=rmCreateTerrainDistanceConstraint("avoid impassable land", "land", false, 8.0);
   int shortAvoidImpassableLand=rmCreateTerrainDistanceConstraint("short avoid impassable land", "land", false, 4.0);
   int farAvoidImpassableLand=rmCreateTerrainDistanceConstraint("far avoid impassable land by not so much", "land", false, 8.0);

// Buildings.
   int avoidBuildings=rmCreateTypeDistanceConstraint("avoid buildings", "Building", 20.0);

// Forest constraints.
   int forestObjConstraint=rmCreateTypeDistanceConstraint("forest obj", "all", 6.0);
   int mixedOakForestConstraint=rmCreateClassDistanceConstraint("mixed oak forest v forest", rmClassID("oak forest"), 5.0);
   int oakForestConstraint=rmCreateClassDistanceConstraint("oak forest v forest", rmClassID("oak forest"), 8.0);
   int pineForestConstraint=rmCreateClassDistanceConstraint("pine forest v forest", rmClassID("pine forest"), 10.0);
   int forestSettleConstraint=rmCreateClassDistanceConstraint("forest settle", rmClassID("starting settlement"), 20.0);
   int farforestSettleConstraint=rmCreateClassDistanceConstraint("far forest settle", rmClassID("far settlement"), 10.0);
   int forestPlayerConstraint=rmCreateClassDistanceConstraint("forest stay away from players", classPlayer, 5);

// Text
   rmSetStatusText("",0.20);

// ----- Define objects.

// Close Objects.

// Starting settlement.
   int startingSettlementID=rmCreateObjectDef("starting settlement");
   rmAddObjectDefItem(startingSettlementID, "Settlement Level 1", 1, 0.0);
   rmAddObjectDefToClass(startingSettlementID, rmClassID("starting settlement"));
   rmSetObjectDefMinDistance(startingSettlementID, 0.0);
   rmSetObjectDefMaxDistance(startingSettlementID, 0.0);

// Towers.
   int startingTowerID=rmCreateObjectDef("Starting tower");
   rmAddObjectDefItem(startingTowerID, "tower", 1, 0.0);
   rmSetObjectDefMinDistance(startingTowerID, 22.0);
   rmSetObjectDefMaxDistance(startingTowerID, 28.0);
   rmAddObjectDefConstraint(startingTowerID, avoidTower);
   rmAddObjectDefConstraint(startingTowerID, avoidImpassableLand);
   rmAddObjectDefConstraint(startingTowerID, avoidConnection);

// Gold.
   int startingGoldID=rmCreateObjectDef("Starting gold");
   rmAddObjectDefItem(startingGoldID, "Gold mine small", 1, 0.0);
   rmSetObjectDefMinDistance(startingGoldID, 20.0);
   rmSetObjectDefMaxDistance(startingGoldID, 25.0);
   rmAddObjectDefConstraint(startingGoldID, shortAvoidGold);
   rmAddObjectDefConstraint(startingGoldID, avoidImpassableLand);
   rmAddObjectDefConstraint(startingGoldID, avoidConnection);

// Berries.
   int closeBerriesID=rmCreateObjectDef("close berries");
   rmAddObjectDefItem(closeBerriesID, "berry bush", rmRandInt(4,8), 4.0);
   rmSetObjectDefMinDistance(closeBerriesID, 15.0);
   rmSetObjectDefMaxDistance(closeBerriesID, 30.0);
   rmAddObjectDefConstraint(closeBerriesID, avoidImpassableLand);
   rmAddObjectDefConstraint(closeBerriesID, shortAvoidGold);
   rmAddObjectDefConstraint(closeBerriesID, avoidFood);
   rmAddObjectDefConstraint(closeBerriesID, avoidConnection);

// Chickens.
   int closeChickensID=rmCreateObjectDef("close birdies");
   rmAddObjectDefItem(closeChickensID, "chicken", rmRandInt(4,8), 4.0);
   rmSetObjectDefMinDistance(closeChickensID, 15.0);
   rmSetObjectDefMaxDistance(closeChickensID, 30.0);
   rmAddObjectDefConstraint(closeChickensID, avoidFood);
   rmAddObjectDefConstraint(closeChickensID, forestObjConstraint);   
   rmAddObjectDefConstraint(closeChickensID, shortAvoidGold);

// Goats. 
   int closeGoatsID=rmCreateObjectDef("close goats");
   rmAddObjectDefItem(closeGoatsID, "goat", rmRandInt(4,8), 2.0);
   rmSetObjectDefMinDistance(closeGoatsID, 15.0);
   rmSetObjectDefMaxDistance(closeGoatsID, 30.0);
   rmAddObjectDefConstraint(closeGoatsID, avoidFood);
   rmAddObjectDefConstraint(closeGoatsID, shortAvoidGold);
   rmAddObjectDefConstraint(closeGoatsID, avoidImpassableLand);

// Trees.
   int stragglerTreeID=rmCreateObjectDef("straggler tree");
   rmAddObjectDefItem(stragglerTreeID, "oak tree", 1, 0.0);
   rmSetObjectDefMinDistance(stragglerTreeID, 12.0);
   rmSetObjectDefMaxDistance(stragglerTreeID, 15.0);

// Medium Objects.

// Gold.
   int mediumGoldID=rmCreateObjectDef("medium gold");
   rmAddObjectDefItem(mediumGoldID, "Gold mine", 1, 0.0);
   rmSetObjectDefMinDistance(mediumGoldID, 40.0);
   rmSetObjectDefMaxDistance(mediumGoldID, 60.0);
   rmAddObjectDefConstraint(mediumGoldID, shortAvoidGold);
   rmAddObjectDefConstraint(mediumGoldID, edgeConstraint);
   rmAddObjectDefConstraint(mediumGoldID, shortAvoidSettlement);
   rmAddObjectDefConstraint(mediumGoldID, avoidImpassableLand);
   rmAddObjectDefConstraint(mediumGoldID, avoidConnection);

// Far Objects.

// Settlement.
   int farSettlementID=rmCreateObjectDef("far settlement");
   rmAddObjectDefItem(farSettlementID, "Settlement", 1, 0.0);
   rmAddObjectDefToClass(farSettlementID, rmClassID("far settlement"));
   rmAddObjectDefConstraint(farSettlementID, edgeConstraint);
   rmAddObjectDefConstraint(farSettlementID, farAvoidSettlement);
   rmAddObjectDefConstraint(farSettlementID, farAvoidImpassableLand);
   rmAddObjectDefConstraint(farSettlementID, forestObjConstraint);
   rmAddObjectDefConstraint(farSettlementID, avoidConnection);

// Gold.
   int farGoldID=rmCreateObjectDef("far gold");
   rmAddObjectDefItem(farGoldID, "Gold mine", 2, 0.0);
   rmSetObjectDefMinDistance(farGoldID, 70.0);
   rmSetObjectDefMaxDistance(farGoldID, 160.0);
   rmAddObjectDefConstraint(farGoldID, shortAvoidGold);
   rmAddObjectDefConstraint(farGoldID, avoidImpassableLand);
   rmAddObjectDefConstraint(farGoldID, shortAvoidSettlement);
   rmAddObjectDefConstraint(farGoldID, farStartingSettleConstraint);
   rmAddObjectDefConstraint(farGoldID, forestObjConstraint);
   rmAddObjectDefConstraint(farGoldID, avoidConnection);

// Berries.
   int farBerriesID=rmCreateObjectDef("far berries");
   rmAddObjectDefItem(farBerriesID, "berry bush", rmRandInt(4,8), 4.0);
   rmSetObjectDefMinDistance(farBerriesID, 30);
   rmSetObjectDefMaxDistance(farBerriesID, 150);
   rmAddObjectDefConstraint(farBerriesID, playerConstraint);
   rmAddObjectDefConstraint(farBerriesID, avoidImpassableLand);
   rmAddObjectDefConstraint(farBerriesID, avoidGold);
   rmAddObjectDefConstraint(farBerriesID, avoidConnection);
   rmAddObjectDefConstraint(farBerriesID, forestObjConstraint);

// Goats.
   int farGoatsID=rmCreateObjectDef("far goats");
   rmAddObjectDefItem(farGoatsID, "goat", rmRandInt(2,4), 1.0);
   rmSetObjectDefMinDistance(farGoatsID, 80.0);
   rmSetObjectDefMaxDistance(farGoatsID, 150.0);
   rmAddObjectDefConstraint(farGoatsID, playerConstraint);
   rmAddObjectDefConstraint(farGoatsID, shortAvoidImpassableLand);
   rmAddObjectDefConstraint(farGoatsID, avoidGold);
   rmAddObjectDefConstraint(farGoatsID, forestObjConstraint);

// Fish.
   int fishID=rmCreateObjectDef("fish");
   rmAddObjectDefItem(fishID, "fish - mahi", 3, 0.0);
   rmSetObjectDefMinDistance(fishID, 0.0);
   rmSetObjectDefMaxDistance(fishID, 1000.0);
   rmAddObjectDefConstraint(fishID, fishVsFishID);
   rmAddObjectDefConstraint(fishID, fishLand);

// Predators
   int farPredatorID=rmCreateObjectDef("far predator");
      if(rmRandFloat(0,1)<0.50)
         rmAddObjectDefItem(farPredatorID, "wolf", rmRandInt(1,3), 4.0);
      else
         rmAddObjectDefItem(farPredatorID, "bear", rmRandInt(1,2), 4.0);
   rmSetObjectDefMinDistance(farPredatorID, 50.0);
   rmSetObjectDefMaxDistance(farPredatorID, 100.0);
   rmAddObjectDefConstraint(farPredatorID, avoidPredator);
   rmAddObjectDefConstraint(farPredatorID, farStartingSettleConstraint);
   rmAddObjectDefConstraint(farPredatorID, avoidImpassableLand);
   rmAddObjectDefConstraint(farPredatorID, forestObjConstraint);

// Bonus huntable
   int bonusHuntableID=rmCreateObjectDef("bonus huntable");
   float huntSpeciesChance=rmRandFloat(0,1);
      if(huntSpeciesChance<.33)
         rmAddObjectDefItem(bonusHuntableID, "boar", rmRandInt(3,5), 2.0);
      else if(rmRandFloat(0,1)<0.50)
         rmAddObjectDefItem(bonusHuntableID, "deer", rmRandInt(3,5), 2.0);
      else
         rmAddObjectDefItem(bonusHuntableID, "caribou", rmRandInt(3,5), 2.0);
   rmSetObjectDefMinDistance(bonusHuntableID, 0.0);
   rmSetObjectDefMaxDistance(bonusHuntableID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(bonusHuntableID, avoidBonusHuntable);
   rmAddObjectDefConstraint(bonusHuntableID, avoidHuntable);
   rmAddObjectDefToClass(bonusHuntableID, classBonusHuntable);
   rmAddObjectDefConstraint(bonusHuntableID, farStartingSettleConstraint);
   rmAddObjectDefConstraint(bonusHuntableID, avoidImpassableLand);
   rmAddObjectDefConstraint(bonusHuntableID, forestObjConstraint);

// Trees.
   int randomTreeID=rmCreateObjectDef("random tree");
      if(rmRandFloat(0,1)<0.33)
         rmAddObjectDefItem(randomTreeID, "oak tree", 2, 0.0);
      else if(rmRandFloat(0,1)<0.66)
         rmAddObjectDefItem(randomTreeID, "oak autumn", 2, 0.0);
      else
         rmAddObjectDefItem(randomTreeID, "pine", 2, 0.0);
   rmSetObjectDefMinDistance(randomTreeID, 0.0);
   rmSetObjectDefMaxDistance(randomTreeID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(randomTreeID, rmCreateTypeDistanceConstraint("random tree", "all", 1.0));
   rmAddObjectDefConstraint(randomTreeID, shortAvoidSettlement);
   rmAddObjectDefConstraint(randomTreeID, avoidImpassableLand);
   rmAddObjectDefConstraint(randomTreeID, avoidConnection);

// Birds.
   int farhawkID=rmCreateObjectDef("far hawks");
   rmAddObjectDefItem(farhawkID, "hawk", 1, 0.0);
   rmSetObjectDefMinDistance(farhawkID, 0.0);
   rmSetObjectDefMaxDistance(farhawkID, rmXFractionToMeters(0.5));

// Relics.
   int relicID=rmCreateObjectDef("relic");
   rmAddObjectDefItem(relicID, "relic", 1, 0.0);
   rmSetObjectDefMinDistance(relicID, 60.0);
   rmSetObjectDefMaxDistance(relicID, 150.0);
   rmAddObjectDefConstraint(relicID, edgeConstraint);
   rmAddObjectDefConstraint(relicID, rmCreateTypeDistanceConstraint("relic vs relic", "relic", 70.0));
   rmAddObjectDefConstraint(relicID, farStartingSettleConstraint);
   rmAddObjectDefConstraint(relicID, avoidImpassableLand);

// Text
   rmSetStatusText("",0.40);

// ----- Areas.

// Player areas.
   rmSetTeamSpacingModifier(0.75);
   rmPlacePlayersCircular(0.3, 0.4, rmDegreesToRadians(5.0));

// Set up player areas.
   float playerFraction=rmAreaTilesToFraction(3000);
   for(i=1; <cNumberPlayers)
   {
// Create the area.
   int id=rmCreateArea("Player"+i);

// Assign to the player.
   rmSetPlayerArea(i, id);

// Set the size.
   rmSetAreaSize(id, 0.9*playerFraction, 1.1*playerFraction);

   rmAddAreaToClass(id, classPlayer);
   rmSetAreaWarnFailure(id, false);
   rmSetAreaMinBlobs(id, 1);
   rmSetAreaMaxBlobs(id, 5);
   rmSetAreaMinBlobDistance(id, 16.0);
   rmSetAreaMaxBlobDistance(id, 40.0);
   rmSetAreaCoherence(id, 0.0);

// Add constraints.
   rmAddAreaConstraint(id, playerConstraint);

// Set the location.
   rmSetAreaLocPlayer(id, i);

// Set type.    
       rmSetAreaTerrainType(id, "GrassA");
   }

// Build the areas.
   rmBuildAllAreas();

// Beautification sub area.
   for(i=1; <cNumberPlayers)
   {
      int id2=rmCreateArea("Player inner"+i, rmAreaID("player"+i));
      rmSetAreaSize(id2, rmAreaTilesToFraction(200), rmAreaTilesToFraction(300));
      rmSetAreaLocPlayer(id2, i);
      rmSetAreaTerrainType(id2, "GrassDirt25");
      rmSetAreaMinBlobs(id2, 3);
      rmSetAreaMaxBlobs(id2, 7);
      rmSetAreaMinBlobDistance(id2, 16.0);
      rmSetAreaMaxBlobDistance(id2, 40.0);
      rmSetAreaCoherence(id2, 0.0);

      rmBuildArea(id2); 
   } 

// Beautification sub area.
      for(i=1; <cNumberPlayers*20)
   {
      int id4=rmCreateArea("Grass patch 2 "+i);
      rmSetAreaSize(id4, rmAreaTilesToFraction(5), rmAreaTilesToFraction(16));
      rmSetAreaTerrainType(id4, "GrassB");
      rmSetAreaMinBlobs(id4, 1);
      rmSetAreaMaxBlobs(id4, 5);
      rmSetAreaWarnFailure(id4, false);
      rmSetAreaMinBlobDistance(id4, 16.0);
      rmSetAreaMaxBlobDistance(id4, 40.0);
      rmSetAreaCoherence(id4, 0.0);

      rmBuildArea(id4);
   }

// Beautification sub area.
   for(i=1; <cNumberPlayers*12)
   {
      int id5=rmCreateArea("Grass patch 3 "+i);
      rmSetAreaSize(id5, rmAreaTilesToFraction(10), rmAreaTilesToFraction(30));
      rmSetAreaTerrainType(id5, "GrassDirt25");
      rmSetAreaMinBlobs(id5, 3);
      rmSetAreaMaxBlobs(id5, 7);
      rmSetAreaWarnFailure(id5, false);
      rmSetAreaMinBlobDistance(id5, 16.0);
      rmSetAreaMaxBlobDistance(id5, 40.0);
      rmSetAreaCoherence(id5, 0.0);

      rmBuildArea(id5);
   }

// Water.
   int numPond=rmRandInt(2,4);
   for(i=0; <numPond)
   {
      int smallPondID=rmCreateArea("small pond"+i);
      rmSetAreaSize(smallPondID, rmAreaTilesToFraction(600), rmAreaTilesToFraction(600));
      rmSetAreaWaterType(smallPondID, "mediterranean sea");
      rmSetAreaMinBlobs(smallPondID, 1);
      rmSetAreaMaxBlobs(smallPondID, 1);
      rmAddAreaToClass(smallPondID, classPond);
      rmAddAreaConstraint(smallPondID, pondConstraint);
      rmAddAreaConstraint(smallPondID, playerConstraint);
      rmSetAreaWarnFailure(smallPondID, false);
      rmBuildArea(smallPondID);
   }

// Create team connections
      id=rmCreateConnection("team connection");
      rmSetConnectionType(id, cConnectAllies, false, 1.0);
      rmSetConnectionWarnFailure(id, false);
      rmSetConnectionWidth(id, 5);
      rmSetConnectionCoherence(id,0);
      rmSetConnectionHeightBlend(id, 2.0);
      rmSetConnectionSmoothDistance(id, 3.0);
      rmSetConnectionPositionVariance(id, 1);
      rmAddConnectionTerrainReplacement(id, "GrassA", "GreekRoadA"); 
      rmAddConnectionTerrainReplacement(id, "GrassDirt25", "GreekRoadA"); 
      rmAddConnectionTerrainReplacement(id, "GrassB", "GreekRoadA"); 
      rmAddConnectionToClass(id, classConnection);
      rmBuildConnection(id);

// Create enemy connections
      id=rmCreateConnection("enemy connection");
      rmSetConnectionType(id, cConnectEnemies, false, 0.5);
      rmSetConnectionWarnFailure(id, false);
      rmSetConnectionWidth(id, 1, 2);
      rmSetConnectionCoherence(id,0);
      rmSetConnectionHeightBlend(id, 2.0);
      rmSetConnectionSmoothDistance(id, 3.0);
      rmSetConnectionPositionVariance(id, 1);
      rmAddConnectionToClass(id, classConnection);
      rmBuildConnection(id);

// Text
   	rmSetStatusText("",0.60);

// ----- Objects.

// Close Objects.

// Place starting settlements.
   rmPlaceObjectDefPerPlayer(startingSettlementID, true);

// Settlements.
   for(i=1; <cNumberPlayers)
   rmPlaceObjectDefInArea(farSettlementID, 0, rmAreaID("player"+i), 3);  

// Towers.
   rmPlaceObjectDefPerPlayer(startingTowerID, true, 4);

// Straggler trees.
   rmPlaceObjectDefPerPlayer(stragglerTreeID, false, rmRandInt(8,12));

// Gold.
   rmPlaceObjectDefPerPlayer(startingGoldID, false, 1);

// Berries.
      rmPlaceObjectDefPerPlayer(closeBerriesID, false, 1);

// Chickens.
    rmPlaceObjectDefPerPlayer(closeChickensID, false, 1);

// Goats
   rmPlaceObjectDefPerPlayer(closeGoatsID, false, 1);
   
// Elevation
   int numTries=6*cNumberNonGaiaPlayers;
   int failCount=0;
   for(i=0; <numTries)
   {
      int elevID=rmCreateArea("elev"+i);
      rmSetAreaSize(elevID, rmAreaTilesToFraction(15), rmAreaTilesToFraction(80));
      rmSetAreaLocation(elevID, rmRandFloat(0.0, 1.0), rmRandFloat(0.0, 1.0));
      rmSetAreaWarnFailure(elevID, false);
      rmSetAreaBaseHeight(elevID, rmRandFloat(3.0, 4.0));
      rmSetAreaMinBlobs(elevID, 5);
      rmSetAreaMaxBlobs(elevID, 10);
      rmSetAreaMinBlobDistance(elevID, 16.0);
      rmSetAreaMaxBlobDistance(elevID, 40.0);
      rmSetAreaCoherence(elevID, 0.0);
      rmAddAreaConstraint(elevID, avoidBuildings);
      rmAddAreaConstraint(elevID, avoidImpassableLand);
      if(rmBuildArea(elevID)==false)
      {
       failCount++;
      if(failCount==8)
       break;
      }
      else
       failCount=0;
   }

// Slight elevation
   numTries=7*cNumberNonGaiaPlayers;
   failCount=0;
   for(i=0; <numTries)
   {
      elevID=rmCreateArea("wrinkle"+i);
      rmSetAreaSize(elevID, rmAreaTilesToFraction(100), rmAreaTilesToFraction(200));
      rmSetAreaLocation(elevID, rmRandFloat(0.0, 1.0), rmRandFloat(0.0, 1.0));
      rmSetAreaWarnFailure(elevID, false);
      rmSetAreaBaseHeight(elevID, rmRandFloat(1.0, 3.0));
      rmSetAreaMinBlobs(elevID, 5);
      rmSetAreaMaxBlobs(elevID, 10);
      rmSetAreaMinBlobDistance(elevID, 16.0);
      rmSetAreaMaxBlobDistance(elevID, 20.0);
      rmSetAreaCoherence(elevID, 0.0);
      rmAddAreaConstraint(elevID, avoidBuildings);
      rmAddAreaConstraint(elevID, avoidImpassableLand);   
      if(rmBuildArea(elevID)==false)
      {
       failCount++;
      if(failCount==8)
       break;
      }
      else
       failCount=0;
   } 

// Text.
   rmSetStatusText("",0.80);

// Forest 1.
   failCount=0;
   numTries=10*cNumberNonGaiaPlayers;
   for(i=0; <numTries)
   {
      int forestOakID=rmCreateArea("oak forest"+i);
      rmSetAreaSize(forestOakID, rmAreaTilesToFraction(250), rmAreaTilesToFraction(300));
      rmSetAreaWarnFailure(forestOakID, false); 
      rmSetAreaForestType(forestOakID, "oak forest");
      rmSetAreaMinBlobs(forestOakID, 4);
      rmSetAreaMaxBlobs(forestOakID, 8);
      rmSetAreaMinBlobDistance(forestOakID, 16);
      rmSetAreaMaxBlobDistance(forestOakID, 40);
      rmSetAreaCoherence(forestOakID, 0.0);
      //rmAddAreaToClass(forestOakID, classOakForest);
      rmAddAreaConstraint(forestOakID, forestPlayerConstraint);
      rmAddAreaConstraint(forestOakID, avoidConnection);

      if(rmBuildArea(forestOakID)==false)
      {
// Stop trying once we fail 3 times in a row.
       failCount++;
      if(failCount==3)
       break;
      }
      else
       failCount=0;
   }

// Forest 2.
   failCount=0;
   numTries=10*cNumberNonGaiaPlayers;
   for(i=0; <numTries)
   {
      int forestMixedOakID=rmCreateArea("mixed oak forest"+i);
      rmSetAreaSize(forestMixedOakID, rmAreaTilesToFraction(300), rmAreaTilesToFraction(350));
      rmSetAreaWarnFailure(forestMixedOakID, false); 
      rmSetAreaForestType(forestMixedOakID, "mixed oak forest");
      rmSetAreaMinBlobs(forestMixedOakID, 4);
      rmSetAreaMaxBlobs(forestMixedOakID, 8);
      rmSetAreaMinBlobDistance(forestMixedOakID, 16);
      rmSetAreaMaxBlobDistance(forestMixedOakID, 40);
      rmSetAreaCoherence(forestMixedOakID, 0.0);
      rmAddAreaToClass(forestMixedOakID, classMixedOakForest);
      rmAddAreaConstraint(forestMixedOakID, forestPlayerConstraint);
      rmAddAreaConstraint(forestMixedOakID, avoidConnection);
      //rmAddAreaConstraint(forestMixedOakID, oakForestConstraint);
      rmAddAreaConstraint(forestMixedOakID, pineForestConstraint);

      if(rmBuildArea(forestMixedOakID)==false)
      {
// Stop trying once we fail 3 times in a row.
       failCount++;
      if(failCount==3)
       break;
      }
      else
       failCount=0;
   }

// Forest 3.
   failCount=0;
   numTries=10*cNumberNonGaiaPlayers;
   for(i=0; <numTries)
   {
      int forestPineID=rmCreateArea("pine forest"+i);
      rmSetAreaSize(forestPineID, rmAreaTilesToFraction(250), rmAreaTilesToFraction(350));
      rmSetAreaWarnFailure(forestPineID, false); 
      rmSetAreaForestType(forestPineID, "pine forest");
      rmSetAreaMinBlobs(forestPineID, 4);
      rmSetAreaMaxBlobs(forestPineID, 8);
      rmSetAreaMinBlobDistance(forestPineID, 16);
      rmSetAreaMaxBlobDistance(forestPineID, 40);
      rmSetAreaCoherence(forestPineID, 0.0);
      rmAddAreaToClass(forestPineID, classPineForest);
      rmAddAreaConstraint(forestPineID, forestPlayerConstraint);
      rmAddAreaConstraint(forestPineID, avoidConnection);
      rmAddAreaConstraint(forestPineID, oakForestConstraint);
      rmAddAreaConstraint(forestPineID, mixedOakForestConstraint);

      if(rmBuildArea(forestPineID)==false)
      {
// Stop trying once we fail 3 times in a row.
       failCount++;
      if(failCount==3)
       break;
      }
      else
       failCount=0;
   }

// Medium Objects.

// Gold.
   rmPlaceObjectDefPerPlayer(mediumGoldID, false, rmRandInt(2, 4));

// Far Objects.

// Gold.
   rmPlaceObjectDefPerPlayer(farGoldID, false, rmRandInt(3, 6));

// Berries.
   rmPlaceObjectDefPerPlayer(farBerriesID, false, 1);

// Goats.
   rmPlaceObjectDefAtLoc(farGoatsID, 0, 0.5, 0.5, cNumberNonGaiaPlayers*rmRandInt(1, 2));

// Fish.
   rmPlaceObjectDefAtLoc(fishID, 0, 0.5, 0.5, cNumberNonGaiaPlayers*rmRandInt(2, 4));

// Predators.
   rmPlaceObjectDefPerPlayer(farPredatorID, false, rmRandInt(2, 4));

// Huntable.
   rmPlaceObjectDefAtLoc(bonusHuntableID, 0, 0.5, 0.5, cNumberNonGaiaPlayers*rmRandInt(4, 6));

// Random trees.
   rmPlaceObjectDefAtLoc(randomTreeID, 0, 0.5, 0.5, 30*cNumberNonGaiaPlayers);

// Hawks.
   rmPlaceObjectDefPerPlayer(farhawkID, false, 2); 

// Relics.
   rmPlaceObjectDefPerPlayer(relicID, true, rmRandInt(2, 4));

// Text.
   rmSetStatusText("",1.00);
}
